from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO


class TestMDLObject(TestCase):
    @min_os_level("10.12")
    def testMethods(self):
        self.assertResultIsBOOL(ModelIO.MDLObject.hidden)
        self.assertArgIsBOOL(ModelIO.MDLObject.setHidden_, 0)

        self.assertArgIsBlock(
            ModelIO.MDLObject.enumerateChildObjectsOfClass_root_usingBlock_stopPointer_,
            2,
            b"v@o^Z",
        )
        self.assertArgHasType(
            ModelIO.MDLObject.enumerateChildObjectsOfClass_root_usingBlock_stopPointer_,
            3,
            b"N^Z",
        )  # XXX: Undocumented



from PyObjCTools.TestSupport import TestCase
import ModelIO


class TestMDLCamera(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLCameraProjectionPerspective, 0)
        self.assertEqual(ModelIO.MDLCameraProjectionOrthographic, 1)

    def testMethods(self):
        self.assertArgIsBOOL(ModelIO.MDLCamera.frameBoundingBox_setNearAndFar_, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import ModelIO


class TestMDLVoxelArray(TestCase):
    @expectedFailure
    def testStructs(self):
        self.fail("MDLVoxelIndexExtent contains SIMD types")

    def testMethods(self):
        self.assertResultIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            1,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            2,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            3,
        )
        self.assertArgIsBOOL(
            ModelIO.MDLVoxelArray.voxelExistsAtIndex_allowAnyX_allowAnyY_allowAnyZ_allowAnyShell_,
            3,
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(ModelIO.MDLVoxelArray.isValidSignedShellField)



# This file is generated by objective.metadata
#
# Last update: Sun Jul 11 21:42:49 2021
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
misc.update(
    {
        "MDLAxisAlignedBoundingBox": objc.createStructType(
            "MDLAxisAlignedBoundingBox",
            b"{_MDLAxisAlignedBoundingBox=}",
            ["maxBounds", "minBounds"],
        ),
        "MDLVoxelIndexExtent": objc.createStructType(
            "MDLVoxelIndexExtent",
            b"{_MDLVoxelIndexExtent=}",
            ["minimumExtent", "maximumExtent"],
        ),
    }
)
constants = """$MDLVertexAttributeAnisotropy$MDLVertexAttributeBinormal$MDLVertexAttributeBitangent$MDLVertexAttributeColor$MDLVertexAttributeEdgeCrease$MDLVertexAttributeJointIndices$MDLVertexAttributeJointWeights$MDLVertexAttributeNormal$MDLVertexAttributeOcclusionValue$MDLVertexAttributePosition$MDLVertexAttributeShadingBasisU$MDLVertexAttributeShadingBasisV$MDLVertexAttributeSubdivisionStencil$MDLVertexAttributeTangent$MDLVertexAttributeTextureCoordinate$kUTType3dObject$kUTTypeAlembic$kUTTypePolygon$kUTTypeStereolithography$kUTTypeUniversalSceneDescription$kUTTypeUniversalSceneDescriptionMobile$"""
enums = """$MDLAnimatedValueInterpolationConstant@0$MDLAnimatedValueInterpolationLinear@1$MDLCameraProjectionOrthographic@1$MDLCameraProjectionPerspective@0$MDLDataPrecisionDouble@2$MDLDataPrecisionFloat@1$MDLDataPrecisionUndefined@0$MDLGeometryTypeLines@1$MDLGeometryTypePoints@0$MDLGeometryTypeQuads@4$MDLGeometryTypeTriangleStrips@3$MDLGeometryTypeTriangles@2$MDLGeometryTypeVariableTopology@5$MDLIndexBitDepthInvalid@0$MDLIndexBitDepthUInt16@16$MDLIndexBitDepthUInt32@32$MDLIndexBitDepthUInt8@8$MDLIndexBitDepthUint16@16$MDLIndexBitDepthUint32@32$MDLIndexBitDepthUint8@8$MDLLightTypeAmbient@1$MDLLightTypeDirectional@2$MDLLightTypeDiscArea@6$MDLLightTypeEnvironment@11$MDLLightTypeLinear@5$MDLLightTypePhotometric@9$MDLLightTypePoint@4$MDLLightTypeProbe@10$MDLLightTypeRectangularArea@7$MDLLightTypeSpot@3$MDLLightTypeSuperElliptical@8$MDLLightTypeUnknown@0$MDLMaterialFaceBack@1$MDLMaterialFaceDoubleSided@2$MDLMaterialFaceFront@0$MDLMaterialMipMapFilterModeLinear@1$MDLMaterialMipMapFilterModeNearest@0$MDLMaterialPropertyTypeBuffer@10$MDLMaterialPropertyTypeColor@4$MDLMaterialPropertyTypeFloat@5$MDLMaterialPropertyTypeFloat2@6$MDLMaterialPropertyTypeFloat3@7$MDLMaterialPropertyTypeFloat4@8$MDLMaterialPropertyTypeMatrix44@9$MDLMaterialPropertyTypeNone@0$MDLMaterialPropertyTypeString@1$MDLMaterialPropertyTypeTexture@3$MDLMaterialPropertyTypeURL@2$MDLMaterialSemanticAmbientOcclusion@22$MDLMaterialSemanticAmbientOcclusionScale@23$MDLMaterialSemanticAnisotropic@7$MDLMaterialSemanticAnisotropicRotation@8$MDLMaterialSemanticBaseColor@0$MDLMaterialSemanticBump@14$MDLMaterialSemanticClearcoat@11$MDLMaterialSemanticClearcoatGloss@12$MDLMaterialSemanticDisplacement@20$MDLMaterialSemanticDisplacementScale@21$MDLMaterialSemanticEmission@13$MDLMaterialSemanticInterfaceIndexOfRefraction@16$MDLMaterialSemanticMaterialIndexOfRefraction@17$MDLMaterialSemanticMetallic@2$MDLMaterialSemanticNone@32768$MDLMaterialSemanticObjectSpaceNormal@18$MDLMaterialSemanticOpacity@15$MDLMaterialSemanticRoughness@6$MDLMaterialSemanticSheen@9$MDLMaterialSemanticSheenTint@10$MDLMaterialSemanticSpecular@3$MDLMaterialSemanticSpecularExponent@4$MDLMaterialSemanticSpecularTint@5$MDLMaterialSemanticSubsurface@1$MDLMaterialSemanticTangentSpaceNormal@19$MDLMaterialSemanticUserDefined@32769$MDLMaterialTextureFilterModeLinear@1$MDLMaterialTextureFilterModeNearest@0$MDLMaterialTextureWrapModeClamp@0$MDLMaterialTextureWrapModeMirror@2$MDLMaterialTextureWrapModeRepeat@1$MDLMeshBufferTypeCustom@3$MDLMeshBufferTypeIndex@2$MDLMeshBufferTypeVertex@1$MDLPrimitiveTypeCapsule@3$MDLPrimitiveTypeCone@2$MDLPrimitiveTypeCube@0$MDLPrimitiveTypeCylinder@4$MDLPrimitiveTypeNone@5$MDLPrimitiveTypeSphere@1$MDLProbePlacementIrradianceDistribution@1$MDLProbePlacementUniformGrid@0$MDLTextureChannelEncodingFloat16@258$MDLTextureChannelEncodingFloat16SR@770$MDLTextureChannelEncodingFloat32@260$MDLTextureChannelEncodingUInt16@2$MDLTextureChannelEncodingUInt24@3$MDLTextureChannelEncodingUInt32@4$MDLTextureChannelEncodingUInt8@1$MDLTextureChannelEncodingUint16@2$MDLTextureChannelEncodingUint24@3$MDLTextureChannelEncodingUint32@4$MDLTextureChannelEncodingUint8@1$MDLTransformOpRotationOrderXYZ@1$MDLTransformOpRotationOrderXZY@2$MDLTransformOpRotationOrderYXZ@3$MDLTransformOpRotationOrderYZX@4$MDLTransformOpRotationOrderZXY@5$MDLTransformOpRotationOrderZYX@6$MDLVertexFormatChar@131073$MDLVertexFormatChar2@131074$MDLVertexFormatChar2Normalized@262146$MDLVertexFormatChar3@131075$MDLVertexFormatChar3Normalized@262147$MDLVertexFormatChar4@131076$MDLVertexFormatChar4Normalized@262148$MDLVertexFormatCharBits@131072$MDLVertexFormatCharNormalized@262145$MDLVertexFormatCharNormalizedBits@262144$MDLVertexFormatFloat@786433$MDLVertexFormatFloat2@786434$MDLVertexFormatFloat3@786435$MDLVertexFormatFloat4@786436$MDLVertexFormatFloatBits@786432$MDLVertexFormatHalf@720897$MDLVertexFormatHalf2@720898$MDLVertexFormatHalf3@720899$MDLVertexFormatHalf4@720900$MDLVertexFormatHalfBits@720896$MDLVertexFormatInt@655361$MDLVertexFormatInt1010102Normalized@659460$MDLVertexFormatInt2@655362$MDLVertexFormatInt3@655363$MDLVertexFormatInt4@655364$MDLVertexFormatIntBits@655360$MDLVertexFormatInvalid@0$MDLVertexFormatPackedBit@4096$MDLVertexFormatShort@393217$MDLVertexFormatShort2@393218$MDLVertexFormatShort2Normalized@524290$MDLVertexFormatShort3@393219$MDLVertexFormatShort3Normalized@524291$MDLVertexFormatShort4@393220$MDLVertexFormatShort4Normalized@524292$MDLVertexFormatShortBits@393216$MDLVertexFormatShortNormalized@524289$MDLVertexFormatShortNormalizedBits@524288$MDLVertexFormatUChar@65537$MDLVertexFormatUChar2@65538$MDLVertexFormatUChar2Normalized@196610$MDLVertexFormatUChar3@65539$MDLVertexFormatUChar3Normalized@196611$MDLVertexFormatUChar4@65540$MDLVertexFormatUChar4Normalized@196612$MDLVertexFormatUCharBits@65536$MDLVertexFormatUCharNormalized@196609$MDLVertexFormatUCharNormalizedBits@196608$MDLVertexFormatUInt@589825$MDLVertexFormatUInt1010102Normalized@593924$MDLVertexFormatUInt2@589826$MDLVertexFormatUInt3@589827$MDLVertexFormatUInt4@589828$MDLVertexFormatUIntBits@589824$MDLVertexFormatUShort@327681$MDLVertexFormatUShort2@327682$MDLVertexFormatUShort2Normalized@458754$MDLVertexFormatUShort3@327683$MDLVertexFormatUShort3Normalized@458755$MDLVertexFormatUShort4@327684$MDLVertexFormatUShort4Normalized@458756$MDLVertexFormatUShortBits@327680$MDLVertexFormatUShortNormalized@458753$MDLVertexFormatUShortNormalizedBits@458752$"""
misc.update({})
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(
        b"MDLAnimatedMatrix4x4",
        b"copyDouble4x4ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"copyFloat4x4ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"double4x4AtTime:",
        {"retval": {"type": b"{_matrix_double4x4=?}"}},
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"float4x4AtTime:",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"getDouble4x4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type": b"^{_matrix_double4x4=?}",
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"getFloat4x4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type": b"^{_matrix_float4x4=?}",
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"resetWithDouble4x4Array:atTimes:count:",
        {"arguments": {2: {"type": b"^{_matrix_double4x4=?}"}}},
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"resetWithDouble4x4Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"resetWithFloat4x4Array:atTimes:count:",
        {"arguments": {2: {"type": b"^{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"resetWithFloat4x4Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"setDouble4x4:atTime:",
        {"arguments": {2: {"type": b"{_matrix_double4x4=?}"}}},
    )
    r(
        b"MDLAnimatedMatrix4x4",
        b"setFloat4x4:atTime:",
        {"arguments": {2: {"type": b"{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"doubleQuaternionAtTime:",
        {"retval": {"type": b"{_simd_quatd=}"}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"floatQuaternionAtTime:",
        {"retval": {"type": b"{_simd_quatf=}"}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"getDoubleQuaternionArray:maxCount:",
        {"arguments": {2: {"type": b"^{_simd_quatd=}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"getFloatQuaternionArray:maxCount:",
        {"arguments": {2: {"type": b"^{_simd_quatf=}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"resetWithDoubleQuaternionArray:atTimes:count:",
        {"arguments": {2: {"type": b"^{_simd_quatd=}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"resetWithFloatQuaternionArray:atTimes:count:",
        {"arguments": {2: {"type": b"^{_simd_quatf=}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"setDoubleQuaternion:atTime:",
        {"arguments": {2: {"type": b"{_simd_quatd=}"}}},
    )
    r(
        b"MDLAnimatedQuaternion",
        b"setFloatQuaternion:atTime:",
        {"arguments": {2: {"type": b"{_simd_quatf=}"}}},
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"getDoubleQuaternionArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_simd_quatd=}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"getDoubleQuaternionArray:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_simd_quatd=}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"getFloatQuaternionArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_simd_quatf=}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"getFloatQuaternionArray:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_simd_quatf=}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"resetWithDoubleQuaternionArray:count:atTimes:count:",
        {
            "arguments": {
                2: {
                    "type": b"^{_simd_quatd=}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                },
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"resetWithFloatQuaternionArray:count:atTimes:count:",
        {
            "arguments": {
                2: {
                    "type": b"^{_simd_quatf=}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                },
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"setDoubleQuaternionArray:count:atTime:",
        {
            "arguments": {
                2: {
                    "type": b"^{_simd_quatd=}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedQuaternionArray",
        b"setFloatQuaternionArray:count:atTime:",
        {
            "arguments": {
                2: {
                    "type": b"^{_simd_quatf=}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"copyDoubleArrayInto:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"copyFloatArrayInto:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"getDoubleArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"getFloatArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"resetWithDoubleArray:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 4},
                3: {"type_modifier": b"n", "c_array_length_in_arg": 4},
            }
        },
    )
    r(
        b"MDLAnimatedScalar",
        b"resetWithFloatArray:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 4},
                3: {"type_modifier": b"n", "c_array_length_in_arg": 4},
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"copyDoubleArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"copyDoubleArrayInto:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"copyFloatArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"copyFloatArrayInto:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"getDoubleArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"getDoubleArray:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"getFloatArray:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"getFloatArray:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"resetWithDoubleArray:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"resetWithFloatArray:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"resetWithFloatArray:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedScalarArray",
        b"setDoubleArray:count:atTime:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MDLAnimatedScalarArray",
        b"setFloatArray:count:atTime:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MDLAnimatedValue",
        b"copyTimesInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedValue",
        b"getTimes:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(b"MDLAnimatedValue", b"isAnimated", {"retval": {"type": "Z"}})
    r(
        b"MDLAnimatedVector2",
        b"copyDouble2ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector2",
        b"copyFloat2ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector2",
        b"getDouble2Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector2",
        b"getFloat2Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector2",
        b"resetWithDouble2Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector2",
        b"resetWithFloat2Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"copyDouble3ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"copyFloat3ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"getDouble3Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"getFloat3Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"resetWithDouble3Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector3",
        b"resetWithFloat3Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"getDouble3Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"getDouble3Array:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"getFloat3Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"getFloat3Array:maxCount:atTime:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"resetWithDouble3Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"resetWithFloat3Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector3Array",
        b"setDouble3Array:count:atTime:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MDLAnimatedVector3Array",
        b"setFloat3Array:count:atTime:",
        {"arguments": {2: {"type_modifier": b"n", "c_array_length_in_arg": 3}}},
    )
    r(
        b"MDLAnimatedVector4",
        b"copyDouble4ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector4",
        b"copyFloat4ArrayInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector4",
        b"getDouble4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector4",
        b"getFloat4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_lengt_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLAnimatedVector4",
        b"resetWithDouble4Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimatedVector4",
        b"resetWithFloat4Array:count:atTimes:count:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_length_in_arg": 3},
                4: {"type_modifier": b"n", "c_array_length_in_arg": 5},
            }
        },
    )
    r(
        b"MDLAnimationBindComponent",
        b"geometryBindTransform",
        {"retval": {"type": b"{_matrix_double4x4=?}"}},
    )
    r(
        b"MDLAnimationBindComponent",
        b"setGeometryBindTransform:",
        {"arguments": {2: {"type": b"{_matrix_double4x4=?}"}}},
    )
    r(
        b"MDLAsset",
        b"boundingBox",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"MDLAsset",
        b"boundingBoxAtTime:",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(b"MDLAsset", b"canExportFileExtension:", {"retval": {"type": b"Z"}})
    r(b"MDLAsset", b"canImportFileExtension:", {"retval": {"type": "Z"}})
    r(b"MDLAsset", b"exportAssetToURL:", {"retval": {"type": "Z"}})
    r(
        b"MDLAsset",
        b"exportAssetToURL:error:",
        {"retval": {"type": "Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"MDLAsset",
        b"initWithURL:bufferAllocator:preserveIndexing:error:",
        {"arguments": {4: {"type": "Z"}, 5: {"type_modifier": b"o"}}},
    )
    r(
        b"MDLAsset",
        b"initWithURL:vertexDescriptor:bufferAllocator:preserveTopology:error:",
        {"arguments": {5: {"type": "Z"}, 6: {"type_modifier": b"o"}}},
    )
    r(
        b"MDLCamera",
        b"frameBoundingBox:setNearAndFar:",
        {
            "arguments": {
                2: {"type": b"{_MDLAxisAlignedBoundingBox=}"},
                3: {"type": "Z"},
            }
        },
    )
    r(b"MDLCamera", b"projectionMatrix", {"retval": {"type": b"{_matrix_float4x4=?}"}})
    r(
        b"MDLMaterialProperty",
        b"initWithName:semantic:matrix4x4:",
        {"arguments": {4: {"type": b"{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLMaterialProperty",
        b"matrix4x4",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLMaterialProperty",
        b"setMatrix4x4:",
        {"arguments": {2: {"type": b"{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLMaterialPropertyNode",
        b"evaluationFunction",
        {
            "retval": {
                "callable": {
                    "retval": {"type": b"v"},
                    "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                }
            }
        },
    )
    r(
        b"MDLMaterialPropertyNode",
        b"initWithInputs:outputs:evaluationFunction:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"MDLMaterialPropertyNode",
        b"setEvaluationFunction:",
        {
            "arguments": {
                2: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}, 1: {"type": b"@"}},
                    }
                }
            }
        },
    )
    r(
        b"MDLMatrix4x4Array",
        b"getDouble4x4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_matrix_double4x4=?}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLMatrix4x4Array",
        b"getFloat4x4Array:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_arg": 3,
                    "type": b"^{_matrix_float4x4=?}",
                    "type_modifier": b"o",
                    "c_array_length_in_result": True,
                }
            }
        },
    )
    r(
        b"MDLMatrix4x4Array",
        b"setDouble4x4Array:count:",
        {
            "arguments": {
                2: {
                    "type": b"^{_matrix_double4x4=?}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLMatrix4x4Array",
        b"setFloat4x4Array:count:",
        {
            "arguments": {
                2: {
                    "type": b"^{_matrix_float4x4=?}",
                    "type_modifier": b"n",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLMesh",
        b"boundingBox",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"MDLMesh",
        b"generateAmbientOcclusionTextureWithQuality:attenuationFactor:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateAmbientOcclusionTextureWithSize:raysPerSample:attenuationFactor:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateAmbientOcclusionVertexColorsWithQuality:attenuationFactor:objectsToConsider:vertexAttributeNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateAmbientOcclusionVertexColorsWithRaysPerSample:attenuationFactor:objectsToConsider:vertexAttributeNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateLightMapTextureWithQuality:lightsToCondider:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateLightMapTextureWithQuality:lightsToConsider:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MDLMesh",
        b"generateLightMapTextureWithTextureSize:lightsToCondider:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"generateLightMapTextureWithTextureSize:lightsToConsider:objectsToConsider:vertexAttributeNamed:materialPropertyNamed:",
        {"retval": {"type": b"Z"}},
    )
    r(
        b"MDLMesh",
        b"generateLightMapVertexColorsWithLightsToConsider:objectsToConsider:vertexAttributeNamed:",
        {"retval": {"type": "Z"}},
    )
    r(
        b"MDLMesh",
        b"initBoxWithExtent:segments:inwardNormals:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initCapsuleWithExtent:cylinderSegments:hemisphereSegments:inwardNormals:geometryType:allocator:",
        {"arguments": {5: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initConeWithExtent:segments:inwardNormals:cap:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}, 5: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initCylinderWithExtent:segments:inwardNormals:topCap:bottomCap:cap:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}, 5: {"type": "Z"}, 6: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initCylinderWithExtent:segments:inwardNormals:topCap:bottomCap:geometryType:allocator:",
        {"arguments": {4: {"type": b"Z"}, 5: {"type": b"Z"}, 6: {"type": b"Z"}}},
    )
    r(
        b"MDLMesh",
        b"initHemisphereWithExtent:segments:inwardNormals:cap:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}, 5: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initIcosahedronWithExtent:inwardNormals:geometryType:allocator:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"MDLMesh",
        b"initIcosahedronWithExtent:inwardNormals:segments:geometryType:allocator:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initMeshWithPrimitive:segments:inwardNormals:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"initSphereWithExtent:segments:inwardNormals:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"makeVerticesUniqueAndReturnError:",
        {"retval": {"type": "Z"}, "arguments": {2: {"type_modifier": b"o"}}},
    )
    r(
        b"MDLMesh",
        b"newBoxWithDimensions:segments:geometryType:inwardNormals:allocator:",
        {"arguments": {5: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newCapsuleWithHeight:radii:radialSegments:verticalSegments:hemisphereSegments:geometryType:inwardNormals:allocator:",
        {"arguments": {8: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newCylinderWithHeight:radii:radialSegments:verticalSegments:geometryType:inwardNormals:allocator:",
        {"arguments": {7: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newEllipsoidWithRadii:radialSegments:verticalSegments:geometryType:inwardNormals:hemisphere:allocator:",
        {"arguments": {6: {"type": "Z"}, 7: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newEllipticalConeWithHeight:radii:radialSegments:verticalSegments:geometryType:inwardNormals:allocator:",
        {"arguments": {7: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newIcosahedronWithRadius:inwardNormals:allocator:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(
        b"MDLMesh",
        b"newIcosahedronWithRadius:inwardNormals:geometryType:allocator:",
        {"arguments": {3: {"type": b"Z"}}},
    )
    r(
        b"MDLMesh",
        b"newMeshWithPrimitive:segments:inwardNormals:geometryType:allocator:",
        {"arguments": {4: {"type": "Z"}}},
    )
    r(b"MDLMeshBufferMap", b"bytes", {"retval": {"c_array_of_variable_length": True}})
    r(
        b"MDLMeshBufferMap",
        b"initWithBytes:deallocator:",
        {
            "arguments": {
                2: {"type_modifier": b"n", "c_array_of_variable_length": True},
                3: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {0: {"type": b"^v"}},
                    }
                },
            }
        },
    )
    r(
        b"MDLMorphDeformer",
        b"copyShapeSetTargetCountsInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLMorphDeformer",
        b"copyShapeSetTargetWeightsInto:maxCount:",
        {
            "arguments": {
                2: {
                    "c_array_length_in_result": True,
                    "type_modifier": b"o",
                    "c_array_length_in_arg": 3,
                }
            }
        },
    )
    r(
        b"MDLMorphDeformer",
        b"initWithTargetShapes:shapeSetTargetWeights:count:shapeSetTargetCounts:count:",
        {
            "arguments": {
                3: {"type_modifier": b"n", "c_array_length_in_arg": 4},
                5: {"type_modifier": b"n", "c_array_length_in_arg": 6},
            }
        },
    )
    r(
        b"MDLNoiseTexture",
        b"initScalarNoiseWithSmoothness:name:textureDimensions:channelCount:channelEncoding:grayScale:",
        {"arguments": {7: {"type": "Z"}}},
    )
    r(
        b"MDLNoiseTexture",
        b"initScalarNoiseWithSmoothness:name:textureDimensions:channelCount:channelEncoding:grayscale:",
        {"arguments": {7: {"type": b"Z"}}},
    )
    r(
        b"MDLObject",
        b"boundingBoxAtTime:",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"MDLObject",
        b"enumerateChildObjectsOfClass:root:usingBlock:stopPointer:",
        {
            "arguments": {
                4: {
                    "callable": {
                        "retval": {"type": b"v"},
                        "arguments": {
                            0: {"type": b"^v"},
                            1: {"type": b"@"},
                            2: {"type": b"o^Z"},
                        },
                    }
                },
                5: {"type": "N^Z"},
            }
        },
    )
    r(b"MDLObject", b"hidden", {"retval": {"type": "Z"}})
    r(b"MDLObject", b"resetsTransform", {"retval": {"type": "Z"}})
    r(b"MDLObject", b"setHidden:", {"arguments": {2: {"type": "Z"}}})
    r(b"MDLObject", b"setResetsTransform:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"MDLSkinDeformer",
        b"copyJointBindTransformsInto:maxCount:",
        {
            "retval": {"type": sel32or64(b"I", b"Q")},
            "arguments": {
                2: {"comment": "matrix_float4x4", "type": "?"},
                3: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(
        b"MDLStereoscopicCamera",
        b"leftProjectionMatrix",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLStereoscopicCamera",
        b"leftViewMatrix",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLStereoscopicCamera",
        b"rightProjectionMatrix",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLStereoscopicCamera",
        b"rightViewMatrix",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(b"MDLTexture", b"hasAlphaValues", {"retval": {"type": "Z"}})
    r(
        b"MDLTexture",
        b"initWithData:topLeftOrigin:name:dimensions:rowStride:channelCount:channelEncoding:isCube:",
        {"arguments": {3: {"type": "Z"}, 9: {"type": "Z"}}},
    )
    r(b"MDLTexture", b"isCube", {"retval": {"type": "Z"}})
    r(b"MDLTexture", b"setHasAlphaValues:", {"arguments": {2: {"type": "Z"}}})
    r(b"MDLTexture", b"setIsCube:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"MDLTexture",
        b"texelDataWithBottomLeftOriginAtMipLevel:create:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(
        b"MDLTexture",
        b"texelDataWithTopLeftOriginAtMipLevel:create:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(b"MDLTexture", b"writeToURL:", {"retval": {"type": "Z"}})
    r(b"MDLTexture", b"writeToURL:level:", {"retval": {"type": "Z"}})
    r(b"MDLTexture", b"writeToURL:type:", {"retval": {"type": "Z"}})
    r(b"MDLTexture", b"writeToURL:type:level:", {"retval": {"type": "Z"}})
    r(
        b"MDLTransform",
        b"initWithMatrix:",
        {"arguments": {2: {"type": b"{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLTransform",
        b"initWithMatrix:resetsTransform:",
        {"arguments": {2: {"type": b"{_matrix_float4x4=?}"}, 3: {"type": "Z"}}},
    )
    r(
        b"MDLTransform",
        b"initWithTransformComponent:resetsTransform:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(
        b"MDLTransform",
        b"rotationMatrixAtTime:",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLTransform",
        b"setMatrix:forTime:",
        {"arguments": {2: {"type": b"{_matrix_float4x4=?}"}}},
    )
    r(
        b"MDLTransformStack",
        b"double4x4AtTime:",
        {"retval": {"type": b"{_matrix_double4x4=?}"}},
    )
    r(
        b"MDLTransformStack",
        b"float4x4AtTime:",
        {"retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(
        b"MDLVertexAttributeData",
        b"dataStart",
        {"retval": {"c_array_of_variable_length": True}},
    )
    r(
        b"MDLVertexAttributeData",
        b"setDataStart:",
        {"arguments": {2: {"c_array_of_variable_length": True}}},
    )
    r(
        b"MDLVoxelArray",
        b"boundingBox",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"MDLVoxelArray",
        b"initWithData:boundingBox:voxelExtent:",
        {"arguments": {3: {"type": b"{_MDLAxisAlignedBoundingBox=}"}}},
    )
    r(b"MDLVoxelArray", b"isValidSignedShellField", {"retval": {"type": "Z"}})
    r(
        b"MDLVoxelArray",
        b"setIsValidSignedShellField:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"MDLVoxelArray",
        b"voxelBoundingBoxAtIndex:",
        {"retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"MDLVoxelArray",
        b"voxelExistsAtIndex:allowAnyX:allowAnyY:allowAnyZ:allowAnyShell:",
        {
            "retval": {"type": "Z"},
            "arguments": {
                3: {"type": "Z"},
                4: {"type": "Z"},
                5: {"type": "Z"},
                6: {"type": "Z"},
            },
        },
    )
    r(
        b"MDLVoxelArray",
        b"voxelIndexExtent",
        {"retval": {"type": b"{_MDLVoxelIndexExtent=}"}},
    )
    r(
        b"MDLVoxelArray",
        b"voxelsWithinExtent:",
        {"arguments": {2: {"type": b"{_MDLVoxelIndexExtent=}"}}},
    )
    r(b"NSObject", b"IsInverseOp", {"required": True, "retval": {"type": b"B"}})
    r(
        b"NSObject",
        b"addObject:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"allocator", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"boundingBox",
        {"required": True, "retval": {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
    )
    r(
        b"NSObject",
        b"canResolveAssetNamed:",
        {"required": True, "retval": {"type": "Z"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"capacity",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"copyJointBindTransformsInto:maxCount:",
        {
            "retval": {"type": sel32or64(b"I", b"Q")},
            "arguments": {
                2: {"comment": "matrix_float4x4", "type": "?"},
                3: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(b"NSObject", b"count", {"required": True, "retval": {"type": b"Q"}})
    r(
        b"NSObject",
        b"double4x4AtTime:",
        {
            "required": True,
            "retval": {"type": b"{_matrix_double4x4=?}"},
            "arguments": {2: {"type": b"d"}},
        },
    )
    r(
        b"NSObject",
        b"fillData:offset:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"float4x4AtTime:",
        {
            "required": True,
            "retval": {"type": b"{_matrix_float4x4=?}"},
            "arguments": {2: {"type": b"d"}},
        },
    )
    r(
        b"NSObject",
        b"globalTransformWithObject:atTime",
        {"arguments": {3: {"type": "d"}}},
    )
    r(
        b"NSObject",
        b"globalTransformWithObject:atTime:",
        {
            "required": False,
            "retval": {"type": b"{_matrix_float4x4=?}"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"d"}},
        },
    )
    r(b"NSObject", b"keyTimes", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"length",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"localTransformAtTime:",
        {
            "required": False,
            "retval": {"type": b"{_matrix_float4x4=?}"},
            "arguments": {2: {"type": "d"}},
        },
    )
    r(b"NSObject", b"map", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"matrix",
        {"required": True, "retval": {"type": b"{_matrix_float4x4=?}"}},
    )
    r(b"NSObject", b"maximumTime", {"required": True, "retval": {"type": "d"}})
    r(b"NSObject", b"minimumTime", {"required": True, "retval": {"type": "d"}})
    r(b"NSObject", b"name", {"required": True, "retval": {"type": b"@"}})
    r(
        b"NSObject",
        b"newBuffer:type:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": sel32or64(b"I", b"Q")},
                3: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(
        b"NSObject",
        b"newBufferFromZone:data:type:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"@"},
                4: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(
        b"NSObject",
        b"newBufferFromZone:length:type:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": sel32or64(b"I", b"Q")},
                4: {"type": sel32or64(b"I", b"Q")},
            },
        },
    )
    r(
        b"NSObject",
        b"newBufferWithData:type:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"newZone:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"newZoneForBuffersWithSize:andType:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"objectAtIndexedSubscript:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": "Q"}}},
    )
    r(b"NSObject", b"objects", {"required": True, "retval": {"type": b"@"}})
    r(b"NSObject", b"primitiveType", {"retval": {"type": sel32or64(b"I", b"Q")}})
    r(
        b"NSObject",
        b"removeObject:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(b"NSObject", b"resetsTransform", {"required": True, "retval": {"type": "Z"}})
    r(
        b"NSObject",
        b"resolveAssetNamed:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setBoundingBox:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"{_MDLAxisAlignedBoundingBox=}"}},
        },
    )
    r(
        b"NSObject",
        b"setLocalTransform:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"{_matrix_float4x4=?}"}},
        },
    )
    r(
        b"NSObject",
        b"setLocalTransform:forTime:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"{_matrix_float4x4=?}"}, 3: {"type": "d"}},
        },
    )
    r(
        b"NSObject",
        b"setMatrix:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"{_matrix_float4x4=?}"}},
        },
    )
    r(b"NSObject", b"setMaximumTime:", {"arguments": {2: {"type": "d"}}})
    r(b"NSObject", b"setMinimumTime:", {"arguments": {2: {"type": "d"}}})
    r(
        b"NSObject",
        b"setName:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setPrimitiveType:",
        {"arguments": {2: {"type": sel32or64(b"I", b"Q")}}},
    )
    r(
        b"NSObject",
        b"setResetsTransform:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": "Z"}}},
    )
    r(
        b"NSObject",
        b"setSphericalHarmonicsLevel:",
        {
            "required": False,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": sel32or64(b"I", b"Q")}},
        },
    )
    r(
        b"NSObject",
        b"sphericalHarmonicsCoefficientsAtPosition:",
        {
            "required": False,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"None"}},
        },
    )
    r(
        b"NSObject",
        b"sphericalHarmonicsLevel",
        {"required": False, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(
        b"NSObject",
        b"type",
        {"required": True, "retval": {"type": sel32or64(b"I", b"Q")}},
    )
    r(b"NSObject", b"zone", {"required": True, "retval": {"type": b"@"}})
finally:
    objc._updatingMetadata(False)
expressions = {}

# END OF FILE



from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import objc
import ModelIO


class TestMDLTypesHelper(ModelIO.NSObject):
    def objectAtIndexedSubscript_(self, i):
        return 1


class TestMDLTypes(TestCase):
    def testConstants(self):
        self.assertIsInstance(ModelIO.kUTTypeAlembic, str)
        self.assertIsInstance(ModelIO.kUTType3dObject, str)
        self.assertIsInstance(ModelIO.kUTTypePolygon, str)
        self.assertIsInstance(ModelIO.kUTTypeStereolithography, str)

        self.assertEqual(ModelIO.MDLDataPrecisionUndefined, 0)
        self.assertEqual(ModelIO.MDLDataPrecisionFloat, 1)
        self.assertEqual(ModelIO.MDLDataPrecisionDouble, 2)

        self.assertEqual(ModelIO.MDLIndexBitDepthInvalid, 0)
        self.assertEqual(ModelIO.MDLIndexBitDepthUInt8, 8)
        self.assertEqual(ModelIO.MDLIndexBitDepthUint8, 8)
        self.assertEqual(ModelIO.MDLIndexBitDepthUInt16, 16)
        self.assertEqual(ModelIO.MDLIndexBitDepthUint16, 16)
        self.assertEqual(ModelIO.MDLIndexBitDepthUInt32, 32)
        self.assertEqual(ModelIO.MDLIndexBitDepthUint32, 32)

        self.assertEqual(ModelIO.MDLGeometryTypePoints, 0)
        self.assertEqual(ModelIO.MDLGeometryTypeLines, 1)
        self.assertEqual(ModelIO.MDLGeometryTypeTriangles, 2)
        self.assertEqual(ModelIO.MDLGeometryTypeTriangleStrips, 3)
        self.assertEqual(ModelIO.MDLGeometryTypeQuads, 4)
        self.assertEqual(ModelIO.MDLGeometryTypeVariableTopology, 5)

        self.assertEqual(ModelIO.MDLProbePlacementUniformGrid, 0)
        self.assertEqual(ModelIO.MDLProbePlacementIrradianceDistribution, 1)

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(ModelIO.kUTTypeUniversalSceneDescription, str)

    @min_os_level("12.0")
    def testConstants11_0(self):
        # Documented as available on 11.0 and later, but not present until 12.0
        self.assertIsInstance(ModelIO.kUTTypeUniversalSceneDescriptionMobile, str)

    def testProtocolObjects(self):
        objc.protocolNamed("MDLNamed")
        objc.protocolNamed("MDLComponent")
        objc.protocolNamed("MDLObjectContainerComponent")

    def testMethods(self):
        self.assertArgHasType(
            TestMDLTypesHelper.objectAtIndexedSubscript_, 0, objc._C_NSUInteger
        )

    @expectedFailure
    def testStructs(self):
        self.fail("MDLAxisAlignedBoundingBox is SIMD type")



from PyObjCTools.TestSupport import TestCase
import objc
import ModelIO


class TestMDLMeshBufferHelper(ModelIO.NSObject):
    def fillData_offset_(self, d, o):
        pass

    def length(self):
        return 1

    def type(self):  # noqa: A003
        return 1

    def capacity(self):
        return 1

    def newZone_(self, c):
        return None

    def newBuffer_type_(self, c, t):
        return None

    def newBufferWithData_type_(self, c, t):
        return None

    def newBufferFromZone_length_type_(self, z, x, t):
        return None

    def newBufferFromZone_data_type_(self, z, d, t):
        return None


class TestMDLMeshBuffer(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLMeshBufferTypeVertex, 1)
        self.assertEqual(ModelIO.MDLMeshBufferTypeIndex, 2)
        self.assertEqual(ModelIO.MDLMeshBufferTypeCustom, 3)

    def testMethods(self):
        self.assertArgIsVariableSize(
            ModelIO.MDLMeshBufferMap.initWithBytes_deallocator_, 0
        )
        self.assertArgIsBlock(
            ModelIO.MDLMeshBufferMap.initWithBytes_deallocator_, 1, b"v"
        )

        self.assertResultIsVariableSize(ModelIO.MDLMeshBufferMap.bytes)

        self.assertArgHasType(
            TestMDLMeshBufferHelper.fillData_offset_, 1, objc._C_NSUInteger
        )
        self.assertResultHasType(TestMDLMeshBufferHelper.length, objc._C_NSUInteger)
        self.assertResultHasType(TestMDLMeshBufferHelper.type, objc._C_NSUInteger)
        self.assertResultHasType(TestMDLMeshBufferHelper.capacity, objc._C_NSUInteger)
        self.assertArgHasType(TestMDLMeshBufferHelper.newZone_, 0, objc._C_NSUInteger)
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBuffer_type_, 0, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBuffer_type_, 1, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBufferWithData_type_, 1, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBufferFromZone_length_type_,
            1,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBufferFromZone_length_type_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestMDLMeshBufferHelper.newBufferFromZone_data_type_, 2, objc._C_NSUInteger
        )

    def testProtocolObjects(self):
        objc.protocolNamed("MDLMeshBuffer")
        objc.protocolNamed("MDLMeshBufferZone")
        objc.protocolNamed("MDLMeshBufferAllocator")



from PyObjCTools.TestSupport import TestCase
import ModelIO


class TestMDLCamera(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLLightTypeUnknown, 0)
        self.assertEqual(ModelIO.MDLLightTypeAmbient, 1)
        self.assertEqual(ModelIO.MDLLightTypeDirectional, 2)
        self.assertEqual(ModelIO.MDLLightTypeSpot, 3)
        self.assertEqual(ModelIO.MDLLightTypePoint, 4)
        self.assertEqual(ModelIO.MDLLightTypeLinear, 5)
        self.assertEqual(ModelIO.MDLLightTypeDiscArea, 6)
        self.assertEqual(ModelIO.MDLLightTypeRectangularArea, 7)
        self.assertEqual(ModelIO.MDLLightTypeSuperElliptical, 8)
        self.assertEqual(ModelIO.MDLLightTypePhotometric, 9)
        self.assertEqual(ModelIO.MDLLightTypeProbe, 10)
        self.assertEqual(ModelIO.MDLLightTypeEnvironment, 11)



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO


class TestMDLTexture(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt8, 1)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint8, 1)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt16, 2)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint16, 2)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt24, 3)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint24, 3)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUInt32, 4)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingUint32, 4)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat16, 0x102)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat16SR, 0x302)
        self.assertEqual(ModelIO.MDLTextureChannelEncodingFloat32, 0x104)

    def testMethods(self):
        self.assertArgIsBOOL(
            ModelIO.MDLTexture.initWithData_topLeftOrigin_name_dimensions_rowStride_channelCount_channelEncoding_isCube_,
            1,
        )
        # self.assertArgIsBOOL(ModelIO.MDLTexture.initWithData_topLeftOrigin_name_dimensions_rowStride_channelCount_channelEncoding_isCube_, 7) # SIMD  # noqa: B950

        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_)
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_type_)

        self.assertArgIsBOOL(
            ModelIO.MDLTexture.texelDataWithTopLeftOriginAtMipLevel_create_, 1
        )
        self.assertArgIsBOOL(
            ModelIO.MDLTexture.texelDataWithBottomLeftOriginAtMipLevel_create_, 1
        )

        self.assertResultIsBOOL(ModelIO.MDLTexture.isCube)
        self.assertArgIsBOOL(ModelIO.MDLTexture.setIsCube_, 0)

        # self.assertArgIsBOOL(ModelIO.MDLNoiseTexture.initScalarNoiseWithSmoothness_name_textureDimensions_channelCount_channelEncoding_grayscale_, 5) # SIMD  # noqa: B950

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(ModelIO.MDLTexture.hasAlphaValues)
        self.assertArgIsBOOL(ModelIO.MDLTexture.setHasAlphaValues_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_level_)
        self.assertResultIsBOOL(ModelIO.MDLTexture.writeToURL_type_level_)



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO


class TestMDLMesh(TestCase):
    def testMethods(self):
        # XXX: Test failure due to ObjC not having a type code for
        # SIMD types.

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newBoxWithDimensions_segments_geometryType_inwardNormals_allocator_, 3)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newEllipsoidWithRadii_radialSegments_verticalSegments_geometryType_inwardNormals_hemisphere_allocator_, 4)  # noqa: B950

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newCylinderWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_, 5)  # noqa: B950

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newEllipticalConeWithHeight_radii_radialSegments_verticalSegments_geometryType_inwardNormals_allocator_, 5)  # noqa: B950

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newIcosahedronWithRadius_inwardNormals_allocator_, 1)

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateAmbientOcclusionTextureWithSize_raysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_)  # noqa: B950

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateAmbientOcclusionTextureWithSize_raysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_)  # noqa: B950

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateAmbientOcclusionVertexColorsWithRaysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_)  # noqa: B950

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateAmbientOcclusionVertexColorsWithRaysPerSample_attenuationFactor_objectsToConsider_vertexAttributeNamed_)  # noqa: B950

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateLightMapTextureWithTextureSize_lightsToConsider_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_)  # noqa: B950

        # self.assertResultIsBOOL(ModelIO.MDLMesh.generateLightMapTextureWithQuality_lightsToConsider_objectsToConsider_vertexAttributeNamed_materialPropertyNamed_)  # noqa: B950

        self.assertResultIsBOOL(
            ModelIO.MDLMesh.generateLightMapVertexColorsWithLightsToConsider_objectsToConsider_vertexAttributeNamed_
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initBoxWithExtent_segments_inwardNormals_geometryType_allocator_, 2)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initSphereWithExtent_segments_inwardNormals_geometryType_allocator_, 2)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initHemisphereWithExtent_segments_inwardNormals_cap_geometryType_allocator_, 2)
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initHemisphereWithExtent_segments_inwardNormals_cap_geometryType_allocator_, 3)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_, 2)  # noqa: B950
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_, 3)  # noqa: B950
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initCylinderWithExtent_segments_inwardNormals_topCap_bottomCap_geometryType_allocator_, 4)  # noqa: B950

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initCapsuleWithExtent_cylinderSegments_hemisphereSegments_inwardNormals_geometryType_allocator_, 3)  # noqa: B950

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initConeWithExtent_segments_inwardNormals_cap_geometryType_allocator_, 2)
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initConeWithExtent_segments_inwardNormals_cap_geometryType_allocator_, 3)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.initIcosahedronWithExtent_inwardNormals_geometryType_allocator_, 1)

        # self.assertArgIsBOOL(ModelIO.MDLMesh.newCapsuleWithHeight_radii_radialSegments_verticalSegments_hemisphereSegments_geometryType_inwardNormals_allocator_, 5)  # noqa: B950

        pass

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(ModelIO.MDLMesh.makeVerticesUniqueAndReturnError_, 0)

        # Vectors..
        # self.assertArgIsBOOL(ModelIO.MDLMesh.initMeshWithPrimitive_segments_inwardNormals_geometryType_allocator_, 2)
        # self.assertArgIsBOOL(ModelIO.MDLMesh.newMeshWithPrimitive_segments_inwardNormals_geometryType_allocator_, 2)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc
import ModelIO


class TestMDLTransformStack(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderXYZ, 1)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderXZY, 2)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderYXZ, 3)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderYZX, 4)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderZXY, 5)
        self.assertEqual(ModelIO.MDLTransformOpRotationOrderZYX, 6)

    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("MDLTransformOp")

        # XXX: Protocol contains matrix types, needs more work



from PyObjCTools.TestSupport import TestCase
import objc
import ModelIO


class TestMDLPrimitiveHelper(ModelIO.NSObject):
    def primitiveType(self):
        return 1

    def setPrimitiveType_(self, v):
        pass


class TestMDLPrimitive(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLPrimitiveTypeCube, 0)
        self.assertEqual(ModelIO.MDLPrimitiveTypeSphere, 1)
        self.assertEqual(ModelIO.MDLPrimitiveTypeCone, 2)
        self.assertEqual(ModelIO.MDLPrimitiveTypeCapsule, 3)
        self.assertEqual(ModelIO.MDLPrimitiveTypeCylinder, 4)
        self.assertEqual(ModelIO.MDLPrimitiveTypeNone, 5)

    # @min_sdk_level('10.13')
    # def testProtocols(self):
    #    objc.protocolNamed('MDLPrimitiveComponent')

    def testMethods(self):
        self.assertResultHasType(
            TestMDLPrimitiveHelper.primitiveType, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLPrimitiveHelper.setPrimitiveType_, 0, objc._C_NSUInteger
        )



from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import ModelIO


class TestMDLValueTypes(TestCase):
    @min_os_level("10.13")
    @expectedFailure
    def testMethodsSIMD(self):
        # SIMD types
        self.assertArgSizeInArg(ModelIO.MDLMatrix4x4Array.setFloat4x4Array_count_, 0, 1)
        self.assertArgIsIn(ModelIO.MDLMatrix4x4Array.setFloat4x4Array_count_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLMatrix4x4Array.setDouble4x4Array_count_, 0, 1
        )
        self.assertArgIsIn(ModelIO.MDLMatrix4x4Array.setDouble4x4Array_count_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLMatrix4x4Array.getFloat4x4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLMatrix4x4Array.getFloat4x4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLMatrix4x4Array.getFloat4x4Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLMatrix4x4Array.getDouble4x4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLMatrix4x4Array.getDouble4x4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLMatrix4x4Array.getDouble4x4Array_maxCount_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import ModelIO


class TestMDLAnimatedValue(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLAnimatedValueInterpolationConstant, 0)
        self.assertEqual(ModelIO.MDLAnimatedValueInterpolationLinear, 1)

    @min_os_level("10.13")
    def testMethods(self):
        self.assertResultIsBOOL(ModelIO.MDLAnimatedValue.isAnimated)

        self.assertArgIsOut(ModelIO.MDLAnimatedValue.getTimes_maxCount_, 0)
        self.assertArgSizeInArg(ModelIO.MDLAnimatedValue.getTimes_maxCount_, 0, 1)
        self.assertArgSizeInResult(ModelIO.MDLAnimatedValue.getTimes_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedValue.copyTimesInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedValue.copyTimesInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedValue.copyTimesInto_maxCount_, 0)

        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.setFloatArray_count_atTime_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.setFloatArray_count_atTime_, 0, 1
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.setFloatArray_count_atTime_, 0
        )

        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.setDoubleArray_count_atTime_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.setDoubleArray_count_atTime_, 0, 1
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.setDoubleArray_count_atTime_, 0
        )

        self.assertArgIsOut(
            ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_atTime_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_atTime_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_atTime_, 0
        )

        self.assertArgIsOut(
            ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_atTime_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_atTime_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_atTime_, 0
        )

        self.assertArgIsOut(ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_, 0)
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedScalarArray.getFloatArray_maxCount_, 0
        )

        self.assertArgIsOut(ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_, 0)
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedScalarArray.getDoubleArray_maxCount_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_atTime_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_atTime_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_atTime_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedScalarArray.copyDoubleArrayInto_maxCount_atTime_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedScalarArray.copyDoubleArrayInto_maxCount_atTime_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedScalarArray.copyDoubleArrayInto_maxCount_atTime_, 0)

        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_, 0
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithFloatArray_count_atTimes_count_, 2
        )

        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_, 0
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_, 0
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalarArray.resetWithDoubleArray_count_atTimes_count_, 2
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedScalarArray.copyFloatArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedValue.copyDoubleArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedValue.copyDoubleArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedValue.copyDoubleArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalar.resetWithFloatArray_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalar.resetWithFloatArray_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalar.resetWithFloatArray_atTimes_count_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalar.resetWithDoubleArray_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalar.resetWithDoubleArray_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedScalar.resetWithDoubleArray_atTimes_count_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedScalar.copyFloatArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedScalar.copyFloatArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedScalar.copyFloatArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedScalar.copyDoubleArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedScalar.copyDoubleArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedScalar.copyDoubleArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(ModelIO.MDLAnimatedScalar.getFloatArray_maxCount_, 0, 1)
        self.assertArgSizeInResult(ModelIO.MDLAnimatedScalar.getFloatArray_maxCount_, 0)
        self.assertArgIsOut(ModelIO.MDLAnimatedScalar.getFloatArray_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedScalar.getDoubleArray_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedScalar.getDoubleArray_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedScalar.getDoubleArray_maxCount_, 0)

    @min_os_level("10.13")
    @expectedFailure
    def testMethodsVector(self):
        # Note: SIMD types like 'vector_float2' are not supported and cannot be tested properly

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.setFloat3Array_count_atTime_, 0, 1
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.setFloat3Array_count_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.setDouble3Array_count_atTime_, 0, 1
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.setDouble3Array_count_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_atTime_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_atTime_, 0
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_atTime_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_atTime_, 0
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.resetWithFloat3Array_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.resetWithFloat3Array_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.resetWithFloat3Array_count_atTimes_count_, 0
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.resetWithFloat3Array_count_atTimes_count_, 2
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.resetWithDouble3Array_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.resetWithDouble3Array_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.resetWithDouble3Array_count_atTimes_count_,
            0,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3Array.resetWithDouble3Array_count_atTimes_count_,
            2,
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector3Array.getFloat3Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_, 0
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedVector3Array.getDouble3Array_maxCount_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.setFloatQuaternionArray_count_atTime_,
            0,
            1,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.setFloatQuaternionArray_count_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.setDoubleQuaternionArray_count_atTime_,
            0,
            1,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.setDoubleQuaternionArray_count_atTime_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_atTime_,
            0,
            1,
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_atTime_,
            0,
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_atTime_,
            0,
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_atTime_,
            0,
            1,
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_atTime_,
            0,
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_atTime_,
            0,
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.resetWithFloatQuaternionArray_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.resetWithFloatQuaternionArray_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.resetWithFloatQuaternionArray_count_atTimes_count_,
            0,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.resetWithFloatQuaternionArray_count_atTimes_count_,
            2,
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.resetWithDoubleQuaternionArray_count_atTimes_count_,
            0,
            1,
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.resetWithDoubleQuaternionArray_count_atTimes_count_,
            2,
            3,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.resetWithDoubleQuaternionArray_count_atTimes_count_,
            0,
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedQuaternionArray.resetWithDoubleQuaternionArray_count_atTimes_count_,
            2,
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_, 0
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedQuaternionArray.getFloatQuaternionArray_maxCount_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_, 0
        )
        self.assertArgIsOut(
            ModelIO.MDLAnimatedQuaternionArray.getDoubleQuaternionArray_maxCount_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.resetWithFloat2Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.resetWithFloat2Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector2.resetWithFloat2Array_atTimes_count_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.resetWithDouble2Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.resetWithDouble2Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector2.resetWithDouble2Array_atTimes_count_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector2.copyFloat2ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector2.copyFloat2ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector2.copyFloat2ArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector2.copyDouble2ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector2.copyDouble2ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector2.copyDouble2ArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.getFloat2Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector2.getFloat2Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector2.getFloat2Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector2.getDouble2Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector2.getDouble2Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector2.getDouble2Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.resetWithFloat3Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.resetWithFloat3Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3.resetWithFloat3Array_atTimes_count_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.resetWithDouble3Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.resetWithDouble3Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector3.resetWithDouble3Array_atTimes_count_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector3.copyFloat3ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector3.copyFloat3ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector3.copyFloat3ArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector3.copyDouble3ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector3.copyDouble3ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector3.copyDouble3ArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.getFloat3Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3.getFloat3Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector3.getFloat3Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector3.getDouble3Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector3.getDouble3Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector3.getDouble3Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.resetWithFloat4Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.resetWithFloat4Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector4.resetWithFloat4Array_atTimes_count_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.resetWithDouble4Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.resetWithDouble4Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector4.resetWithDouble4Array_atTimes_count_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector4.copyFloat4ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector4.copyFloat4ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector4.copyFloat4ArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector4.copyDouble4ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector4.copyDouble4ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector4.copyDouble4ArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.getFloat4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector4.getFloat4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector4.getFloat4Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4.getDouble4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector4.getDouble4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector4.getDouble4Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.resetWithFloat4x4Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.resetWithFloat4x4Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector4x4.resetWithFloat4x4Array_atTimes_count_, 0
        )

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.resetWithDouble4x4Array_atTimes_count_, 0, 2
        )
        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.resetWithDouble4x4Array_atTimes_count_, 1, 2
        )
        self.assertArgIsIn(
            ModelIO.MDLAnimatedVector4x4.resetWithDouble4x4Array_atTimes_count_, 0
        )

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector4x4.copyFloat4x4ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector4x4.copyFloat4x4ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector4x4.copyFloat4x4ArrayInto_maxCount_, 0)

        # self.assertArgSizeInArg(ModelIO.MDLAnimatedVector4x4.copyDouble4x4ArrayInto_maxCount_, 0, 1)
        # self.assertArgSizeInResult(ModelIO.MDLAnimatedVector4x4.copyDouble4x4ArrayInto_maxCount_, 0)
        # self.assertArgIsOut(ModelIO.MDLAnimatedVector4x4.copyDouble4x4ArrayInto_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.getFloat4x4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector4x4.getFloat4x4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector4x4.getFloat4x4Array_maxCount_, 0)

        self.assertArgSizeInArg(
            ModelIO.MDLAnimatedVector4x4.getDouble4x4Array_maxCount_, 0, 1
        )
        self.assertArgSizeInResult(
            ModelIO.MDLAnimatedVector4x4.getDouble4x4Array_maxCount_, 0
        )
        self.assertArgIsOut(ModelIO.MDLAnimatedVector4x4.getDouble4x4Array_maxCount_, 0)



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc
import ModelIO  # noqa: F401


class TestMDLAnimation(TestCase):
    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("MDLJointAnimation")



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import ModelIO


class TestMDLTransformHelper(ModelIO.NSObject):
    def matrix(self):
        return 1

    def setMatrix_(self, v):
        return 1

    def resetsTransform(self):
        return 1

    def setResetsTransform_(self, v):
        return 1

    def minimumTime(self):
        return 1

    def setMinimumTime_(self, v):
        return 1

    def maximumTime(self):
        return 1

    def setMaximumTime_(self, v):
        return 1

    def setLocalTransform_forTime_(self, t, tm):
        pass

    def setLocalTransform_(self, t):
        pass

    def localTransformAtTime_(self, t):
        return 1

    def globalTransformWithObject_atTime_(self, o, t):
        return 1


class TestMDLTransform(TestCase):
    def testMethods(self):
        # self.assertResultHasType(ModelIO.TestMDLTransformHelper.matrix, ...) # SIMD
        # self.assertResultHasType(ModelIO.TestMDLTransformHelper.setMatrix_, 0, ...) # SIMD

        self.assertResultIsBOOL(ModelIO.TestMDLTransformHelper.resetsTransform)
        self.assertArgIsBOOL(ModelIO.TestMDLTransformHelper.setResetsTransform_, 0)

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.minimumTime, objc._C_DBL
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setMinimumTime_, 0, objc._C_DBL
        )

        self.assertResultHasType(
            ModelIO.TestMDLTransformHelper.maximumTime, objc._C_DBL
        )
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setMaximumTime_, 0, objc._C_DBL
        )

        # self.assertArgHasType(ModelIO.TestMDLTransformHelper.setLocalTransform_forTime_, 0, ...) # SIMD
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.setLocalTransform_forTime_, 1, objc._C_DBL
        )

        # self.assertArgHasType(ModelIO.TestMDLTransformHelper.setLocalTransform_, 0, ...) # SIMD

        # self.assertResultHasType(ModelIO.TestMDLTransformHelper.localTransformAtTime_, ...) # SIMD
        self.assertArgHasType(
            ModelIO.TestMDLTransformHelper.localTransformAtTime_, 0, objc._C_DBL
        )

        # self.assertResultHasType(ModelIO.TestMDLTransformHelper.globalTransformWithObject_atTime_, ...) # SIMD
        # self.assertArgHasType(ModelIO.TestMDLTransformHelper.globalTransformWithObject_atTime_, 1, objc._C_DBL)

        # self.assertArgIsHasType(ModelIO.MDLTransform.initWithMatrix_resetsTransform_, 0, ...) # SIMD
        # self.assertArgIsBOOL(ModelIO.MDLTransform.initWithMatrix_resetsTransform_, 1)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBOOL(
            ModelIO.MDLTransform.initWithTransformComponent_resetsTransform_, 1
        )

    def testProtocolObjects(self):
        objc.protocolNamed("MDLTransformComponent")



from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc
import ModelIO


class TestMDLAssetResolverHelper(ModelIO.NSObject):
    def canResolveAssetNamed_(self, name):
        return 1


class TestMDLAssetResolver(TestCase):
    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("MDLAssetResolver")

    def testMethods(self):
        self.assertResultIsBOOL(
            ModelIO.TestMDLAssetResolverHelper.canResolveAssetNamed_
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import ModelIO


class TestMDLCamera(TestCase):
    def testConstants(self):
        self.assertEqual(ModelIO.MDLMaterialSemanticBaseColor, 0)
        self.assertEqual(ModelIO.MDLMaterialSemanticSubsurface, 1)
        self.assertEqual(ModelIO.MDLMaterialSemanticMetallic, 2)
        self.assertEqual(ModelIO.MDLMaterialSemanticSpecular, 3)
        self.assertEqual(ModelIO.MDLMaterialSemanticSpecularExponent, 4)
        self.assertEqual(ModelIO.MDLMaterialSemanticSpecularTint, 5)
        self.assertEqual(ModelIO.MDLMaterialSemanticRoughness, 6)
        self.assertEqual(ModelIO.MDLMaterialSemanticAnisotropic, 7)
        self.assertEqual(ModelIO.MDLMaterialSemanticAnisotropicRotation, 8)
        self.assertEqual(ModelIO.MDLMaterialSemanticSheen, 9)
        self.assertEqual(ModelIO.MDLMaterialSemanticSheenTint, 10)
        self.assertEqual(ModelIO.MDLMaterialSemanticClearcoat, 11)
        self.assertEqual(ModelIO.MDLMaterialSemanticClearcoatGloss, 12)
        self.assertEqual(ModelIO.MDLMaterialSemanticEmission, 13)
        self.assertEqual(ModelIO.MDLMaterialSemanticBump, 14)
        self.assertEqual(ModelIO.MDLMaterialSemanticOpacity, 15)
        self.assertEqual(ModelIO.MDLMaterialSemanticInterfaceIndexOfRefraction, 16)
        self.assertEqual(ModelIO.MDLMaterialSemanticMaterialIndexOfRefraction, 17)
        self.assertEqual(ModelIO.MDLMaterialSemanticObjectSpaceNormal, 18)
        self.assertEqual(ModelIO.MDLMaterialSemanticTangentSpaceNormal, 19)
        self.assertEqual(ModelIO.MDLMaterialSemanticDisplacement, 20)
        self.assertEqual(ModelIO.MDLMaterialSemanticDisplacementScale, 21)
        self.assertEqual(ModelIO.MDLMaterialSemanticAmbientOcclusion, 22)
        self.assertEqual(ModelIO.MDLMaterialSemanticAmbientOcclusionScale, 23)
        self.assertEqual(ModelIO.MDLMaterialSemanticNone, 0x8000)
        self.assertEqual(ModelIO.MDLMaterialSemanticUserDefined, 0x8001)

        self.assertEqual(ModelIO.MDLMaterialPropertyTypeNone, 0)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeString, 1)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeURL, 2)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeTexture, 3)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeColor, 4)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeFloat, 5)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeFloat2, 6)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeFloat3, 7)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeFloat4, 8)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeMatrix44, 9)
        self.assertEqual(ModelIO.MDLMaterialPropertyTypeBuffer, 10)

        self.assertEqual(ModelIO.MDLMaterialTextureWrapModeClamp, 0)
        self.assertEqual(ModelIO.MDLMaterialTextureWrapModeRepeat, 1)
        self.assertEqual(ModelIO.MDLMaterialTextureWrapModeMirror, 2)

        self.assertEqual(ModelIO.MDLMaterialTextureFilterModeNearest, 0)
        self.assertEqual(ModelIO.MDLMaterialTextureFilterModeLinear, 1)

        self.assertEqual(ModelIO.MDLMaterialMipMapFilterModeNearest, 0)
        self.assertEqual(ModelIO.MDLMaterialMipMapFilterModeLinear, 1)

        self.assertEqual(ModelIO.MDLMaterialFaceFront, 0)
        self.assertEqual(ModelIO.MDLMaterialFaceBack, 1)
        self.assertEqual(ModelIO.MDLMaterialFaceDoubleSided, 2)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBlock(
            ModelIO.MDLMaterialPropertyNode.initWithInputs_outputs_evaluationFunction_,
            2,
            b"v@",
        )
        self.assertArgIsBlock(
            ModelIO.MDLMaterialPropertyNode.setEvaluationFunction_, 0, b"v@"
        )
        self.assertResultIsBlock(
            ModelIO.MDLMaterialPropertyNode.evaluationFunction, b"v@"
        )



from PyObjCTools.TestSupport import TestCase
import ModelIO


class TestMDLVertexDescriptor(TestCase):
    def testConstants(self):
        self.assertIsInstance(ModelIO.MDLVertexAttributeAnisotropy, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeBinormal, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeBitangent, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeColor, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeEdgeCrease, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeJointIndices, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeJointWeights, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeNormal, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeOcclusionValue, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributePosition, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeShadingBasisU, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeShadingBasisV, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeSubdivisionStencil, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeTangent, str)
        self.assertIsInstance(ModelIO.MDLVertexAttributeTextureCoordinate, str)

        self.assertEqual(ModelIO.MDLVertexFormatInvalid, 0)
        self.assertEqual(ModelIO.MDLVertexFormatPackedBit, 0x1000)
        self.assertEqual(ModelIO.MDLVertexFormatUCharBits, 0x10000)
        self.assertEqual(ModelIO.MDLVertexFormatCharBits, 0x20000)
        self.assertEqual(ModelIO.MDLVertexFormatUCharNormalizedBits, 0x30000)
        self.assertEqual(ModelIO.MDLVertexFormatCharNormalizedBits, 0x40000)
        self.assertEqual(ModelIO.MDLVertexFormatUShortBits, 0x50000)
        self.assertEqual(ModelIO.MDLVertexFormatShortBits, 0x60000)
        self.assertEqual(ModelIO.MDLVertexFormatUShortNormalizedBits, 0x70000)
        self.assertEqual(ModelIO.MDLVertexFormatShortNormalizedBits, 0x80000)
        self.assertEqual(ModelIO.MDLVertexFormatUIntBits, 0x90000)
        self.assertEqual(ModelIO.MDLVertexFormatIntBits, 0xA0000)
        self.assertEqual(ModelIO.MDLVertexFormatHalfBits, 0xB0000)
        self.assertEqual(ModelIO.MDLVertexFormatFloatBits, 0xC0000)
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar, ModelIO.MDLVertexFormatUCharBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar2, ModelIO.MDLVertexFormatUCharBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar3, ModelIO.MDLVertexFormatUCharBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar4, ModelIO.MDLVertexFormatUCharBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar, ModelIO.MDLVertexFormatCharBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar2, ModelIO.MDLVertexFormatCharBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar3, ModelIO.MDLVertexFormatCharBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar4, ModelIO.MDLVertexFormatCharBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUCharNormalized,
            ModelIO.MDLVertexFormatUCharNormalizedBits | 1,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar2Normalized,
            ModelIO.MDLVertexFormatUCharNormalizedBits | 2,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar3Normalized,
            ModelIO.MDLVertexFormatUCharNormalizedBits | 3,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUChar4Normalized,
            ModelIO.MDLVertexFormatUCharNormalizedBits | 4,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatCharNormalized,
            ModelIO.MDLVertexFormatCharNormalizedBits | 1,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar2Normalized,
            ModelIO.MDLVertexFormatCharNormalizedBits | 2,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar3Normalized,
            ModelIO.MDLVertexFormatCharNormalizedBits | 3,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatChar4Normalized,
            ModelIO.MDLVertexFormatCharNormalizedBits | 4,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort, ModelIO.MDLVertexFormatUShortBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort2, ModelIO.MDLVertexFormatUShortBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort3, ModelIO.MDLVertexFormatUShortBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort4, ModelIO.MDLVertexFormatUShortBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort, ModelIO.MDLVertexFormatShortBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort2, ModelIO.MDLVertexFormatShortBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort3, ModelIO.MDLVertexFormatShortBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort4, ModelIO.MDLVertexFormatShortBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShortNormalized,
            ModelIO.MDLVertexFormatUShortNormalizedBits | 1,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort2Normalized,
            ModelIO.MDLVertexFormatUShortNormalizedBits | 2,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort3Normalized,
            ModelIO.MDLVertexFormatUShortNormalizedBits | 3,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUShort4Normalized,
            ModelIO.MDLVertexFormatUShortNormalizedBits | 4,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShortNormalized,
            ModelIO.MDLVertexFormatShortNormalizedBits | 1,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort2Normalized,
            ModelIO.MDLVertexFormatShortNormalizedBits | 2,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort3Normalized,
            ModelIO.MDLVertexFormatShortNormalizedBits | 3,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatShort4Normalized,
            ModelIO.MDLVertexFormatShortNormalizedBits | 4,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUInt, ModelIO.MDLVertexFormatUIntBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUInt2, ModelIO.MDLVertexFormatUIntBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUInt3, ModelIO.MDLVertexFormatUIntBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUInt4, ModelIO.MDLVertexFormatUIntBits | 4
        )
        self.assertEqual(ModelIO.MDLVertexFormatInt, ModelIO.MDLVertexFormatIntBits | 1)
        self.assertEqual(
            ModelIO.MDLVertexFormatInt2, ModelIO.MDLVertexFormatIntBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatInt3, ModelIO.MDLVertexFormatIntBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatInt4, ModelIO.MDLVertexFormatIntBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatHalf, ModelIO.MDLVertexFormatHalfBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatHalf2, ModelIO.MDLVertexFormatHalfBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatHalf3, ModelIO.MDLVertexFormatHalfBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatHalf4, ModelIO.MDLVertexFormatHalfBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatFloat, ModelIO.MDLVertexFormatFloatBits | 1
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatFloat2, ModelIO.MDLVertexFormatFloatBits | 2
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatFloat3, ModelIO.MDLVertexFormatFloatBits | 3
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatFloat4, ModelIO.MDLVertexFormatFloatBits | 4
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatInt1010102Normalized,
            ModelIO.MDLVertexFormatIntBits | ModelIO.MDLVertexFormatPackedBit | 4,
        )
        self.assertEqual(
            ModelIO.MDLVertexFormatUInt1010102Normalized,
            ModelIO.MDLVertexFormatUIntBits | ModelIO.MDLVertexFormatPackedBit | 4,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import objc
import ModelIO


class TestMDLAssetHelper(ModelIO.NSObject):
    # XXX: vector tytes
    # def boundingBox(self):
    #    return 1

    # XXX: vector tytes
    # def setBoundingBox_(self, v):
    #    pass

    def sphericalHarmonicsLevel(self):
        return 1

    def setSphericalHarmonicsLevel_(self, v):
        pass


class TestMDLAsset(TestCase):
    def testMethods(self):
        self.assertArgIsBOOL(
            ModelIO.MDLAsset.initWithURL_vertexDescriptor_bufferAllocator_preserveTopology_error_,
            3,
        )
        self.assertArgIsOut(
            ModelIO.MDLAsset.initWithURL_vertexDescriptor_bufferAllocator_preserveTopology_error_,
            4,
        )

        self.assertResultIsBOOL(ModelIO.MDLAsset.exportAssetToURL_)

        self.assertResultIsBOOL(ModelIO.MDLAsset.exportAssetToURL_error_)
        self.assertArgIsOut(ModelIO.MDLAsset.exportAssetToURL_error_, 1)

        self.assertResultIsBOOL(ModelIO.MDLAsset.canImportFileExtension_)

        self.assertResultIsBOOL(ModelIO.MDLAsset.canExportFileExtension_)

        # XXX: SIMD types
        # self.assertResultHasType(TestMDLAssetHelper.boundingBox, ...)
        # self.assertArgHasType(TestMDLAssetHelper.setBoundingBox_, 0, ...)
        # self.assertArgHasType(TestMDLAssetHelper.sphericalHarmonicsCoefficientsAtPosition_, 0, ...)

        self.assertResultHasType(
            TestMDLAssetHelper.sphericalHarmonicsLevel, objc._C_NSUInteger
        )
        self.assertArgHasType(
            TestMDLAssetHelper.setSphericalHarmonicsLevel_, 0, objc._C_NSUInteger
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBOOL(
            ModelIO.MDLAsset.initWithURL_bufferAllocator_preserveIndexing_error_, 2
        )
        self.assertArgIsOut(
            ModelIO.MDLAsset.initWithURL_bufferAllocator_preserveIndexing_error_, 3
        )

    def testProtocolObjects(self):
        objc.protocolNamed("MDLLightProbeIrradianceDataSource")
